#! /usr/bin/python


## PopSciColl processor

## Copyright 2014 Tom Brown

## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 3 of the
## License, or (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''
Usage:
Options:
-h, --help
displays this help message
'''

__author__ = "Tom Brown"
__copyright__ = "Copyright 2014 Tom Brown, GNU GPL 3"


#The following gives a choice between using the Python Imaging Library
#(PIL) to convert picture size and using ImageMagick convert from the
#command line.  For MacOSX ImageMagick comes with the MacTeX
#installation.

#graphicsConverter = "PIL"
graphicsConverter = "ImageMagick"


#The following is the Python Imaging Library (PIL)
#on Ubuntu GNU/Linux it was already there
#on a Mac install with "sudo easy_install pil"

if graphicsConverter=="PIL":
    import Image



import os,sys,getopt,re

# http://www.freewisdom.org/projects/python-markdown/
# note that this version of markdown will do work inside span and div
# on a Mac install with command "sudo easy_install Markdown"


import markdown



#for HTML template
from string import Template




statcounterText = """
<script type="text/javascript">
var sc_project=6167973;
var sc_invisible=1;
var sc_security="dc26907c";
</script>

<script type="text/javascript"
src="https://www.statcounter.com/counter/counter.js"></script>
"""


def processThumbnails(text):


    #deal with thumbnails
    while(text.find("[t:") > 0):
        thumbStart = text.find("[t:")
        thumbStop = thumbStart + text[thumbStart:].find("]")

        if thumbStop < 0:
            print "warning, no end to thumbnail!"

        thumbnail = text[thumbStart+3:thumbStop]

        if thumbnail in ["../earth","../life","../universe","../technology"]:
            thumblink = thumbnail + ".html"
        else:
            thumblink = thumbnail + "/index.html"

        if thumbnail[:3]=="../":
            thumbname = thumbnail[3:]
            path = "../"
        else:
            thumbname = thumbnail
            path = ""

        text = text[:thumbStart] + "<a href=\"" + thumblink + "\"><img class=\"thumbnail\" src=\"" + path + "thumbnails/" + thumbname + "-110x70.png\" /></a>" + text[thumbStop+1:]

    return text





def processIndex():

    fullFilePath = "index.mar"
    inputFile = open(fullFilePath, 'r')

    title = inputFile.readline()

    bulk = inputFile.read()

    inputFile.close()

    temp = Template(open("temp.html").read())

    values = {
        "topTitle" : "home",
        "javascript" : "",
        "uplist" : title,
        "up" : "",
        "relativeURL" : "",
        "buttons" : "",
        "bulk" : markdown.markdown(bulk),
        "statcounterText" : statcounterText
        }

    outputFile = open(fullFilePath[:-3] + 'html', 'w')
    outputFile.write(temp.substitute(values))
    outputFile.close()



def processTopLevel(fullFilePath):


    inputFile = open(fullFilePath, 'r')

    title = inputFile.readline()

    bulk=""
    completedHierarchy=""
    for line in inputFile:
        depth = -1
        if line[0]==">":
            depth =  line.rfind(">")
        if depth >= 0:
            if line[depth+1]=="l":
                line = "    "*depth + "* <span class=\"onlyLinks\">" + line[depth+3:-1] + " (only links)</span>\n"
            elif line[depth+1]=="u":
                line = "    "*depth + "* <span class=\"onlyLinks\">" + line[depth+3:-1] + " (unfinished)</span>\n"
            elif line[depth+1]=="f":
                line = "    "*depth + "* <span class=\"finished\">" + line[depth+3:-1] + "</span>\n"
                completedHierarchy+=line[line.find("*"):]
#                completedHierarchy+=line
            else:
                line = "    "*depth + "*" + line[depth+1:]
        bulk+=line

    inputFile.close()


    hierarchyOpt=0
    #option for hierarchies
    if "\"onlyLinks\"" in bulk or "\"finished\"" in bulk:
        hierarchyOpt=1

    if hierarchyOpt:
        minimum = 200
        for line in completedHierarchy.splitlines(True):
            if line.find("*")<minimum:
                minimum=line.find("*")
        newHierarchy = ""
        for line in completedHierarchy.splitlines(True):
            newHierarchy+=line[minimum:]

        #note that here markdown="1" is required for markdown to process inside div, see
        #http://stackoverflow.com/questions/1705895/python-markdown-markdown-inside-html-blocks

        bulk=bulk[:bulk.find("*")] +"""
<div id="completedHierarchy" markdown="1">

<h2>Finished Slideshows</h2>


""" + completedHierarchy + """

</div>


<div id="fullHierarchy"  markdown="1">

<h2>Full Hierarchy</h2>


""" + bulk[bulk.find("*"):] + "\n</div>\n"

    javascript = ""

    upButton = "<div id=\"up\">\n<a href=\"index.html\"><img src=\"arrow-up-50.jpg\" /></a>\n</div>"


    text = """
<div style="margin: 0 0 50px 0;">
<div id="box">
"""
    text+="<h1>" + title[title.rfind(">")+1:] + "</h1>\n"



    bulk = processThumbnails(bulk)


    # see
    # http://freewisdom.org/projects/python-markdown/Table_of_Contents
    # for the table of contents extension
    #
    # for tables syntax see
    # http://michelf.com/projects/php-markdown/extra/#table

#    text+=markdown.markdown(bulk,extensions=["toc(title=" + title[title.rfind(">")+1:] + ")","tables"])

    #note that the extension "extra" is required for markdown to process inside div, see
    #http://stackoverflow.com/questions/1705895/python-markdown-markdown-inside-html-blocks


    text+=markdown.markdown(bulk,extensions=["toc","tables","extra"])

    text+="""\n</div>\n</div>\n"""


    #only do the following replacement for hierarchies
    if hierarchyOpt:
        text = text.replace("<ul>","<ul class=\"hierarchy\">",2)



    #up buttons
    buttons = """
<div id="previousContainer">
<div id="innerPreviousContainer">
<a href="index.html"><img src="arrow-up-80.jpg" /></a>
<ul id="prevList">
<li><a href="index.html">up to home</a></li>
</ul>
</div>
</div>

<div id="nextContainer">
<a href="index.html"><img src="arrow-up-80.jpg" /></a>
<ul id="nextList">
<li><a href="index.html">up to home</a></li>
</ul>
</div>
"""



    temp = Template(open("temp.html").read())

    values = {
        "topTitle" : title[title.find(">")+1:],
        "javascript" : javascript,
        "uplist" : "",
        "up" : upButton,
        "relativeURL" : "",
        "buttons" : buttons,
        "bulk" : text,
        "statcounterText" : statcounterText
        }



    outputFile = open(fullFilePath[:-3] + 'html', 'w')
    outputFile.write(temp.substitute(values))
    outputFile.close()



class image(object):
    def __init__(self,text,dirName,imgWidth):
        self.url = text[:text.find(",")].strip()
        self.caption = text[text.find(",")+1:]
        self.imgWidth = -1
        self.dirName = dirName

        if not os.path.isfile(self.dirName + self.url):
            print "image file " + self.dirName + self.url + " not found!"
            sys.exit(0)

        if imgWidth > 0:
            self.changeWidth(imgWidth)




    def convertPIL(self,newWidth):
        image = Image.open(self.dirName + self.url)
        image.thumbnail((int(newWidth),50000))
        image.save(self.dirName+ self.url[:self.url.rfind(".")] + "-" + newWidth + self.url[self.url.rfind("."):])


    def convertImageMagick(self,newWidth):
        command = "convert -resize " + newWidth  + " " + self.dirName + self.url + " " + self.dirName+ self.url[:self.url.rfind(".")] + "-" + newWidth + self.url[self.url.rfind("."):]
        os.system(command)



    # note that newWidth is a string
    def changeWidth(self,newWidth):
        if not os.path.isfile(self.dirName+self.url[:self.url.rfind(".")] + "-" + newWidth+self.url[self.url.rfind("."):]):
            print "converting " + self.url + " to width " + newWidth
            if graphicsConverter == "PIL":
                self.convertPIL(newWidth)
            elif graphicsConverter == "ImageMagick":
                self.convertImageMagick(newWidth)

        self.imgWidth=newWidth
        self.url = self.url[:self.url.rfind(".")] + "-" + self.imgWidth + self.url[self.url.rfind("."):]



class imageSet(object):
    def __init__(self,text,dirName):

        self.dirName=dirName

        startBrack = text.find("(")
        stopBrack = text.find(")")

        self.imgWidth = -1

        if startBrack > 0:
            #NB self.imgWidth is a string
            self.imgWidth = text[startBrack+1:stopBrack]
            text=text[:startBrack]+text[stopBrack+1:]

        self.images=[]

        if text!="":
            for item in text[4:].split(";"):
                self.images.append(image(item,self.dirName,self.imgWidth))

    def JSlistURLs(self):
        HTML = "["
        for item in self.images:
            HTML+="\"" + item.url+"\","
        return HTML[:-1]+"]"

    def JSlistCaptions(self):
        HTML = "["
        for item in self.images:
            HTML+="\"" + item.caption+"\","
        return HTML[:-1]+"]"


    #NB accepts counter as a string
    def toHTML(self,counter):

        if len(self.images)==1:
            HTML="\n<div class=\"img_box\" style=\"width:" + str(self.imgWidth)+"px;\"><img src=\"" + self.images[0].url + "\" id=\"mainPic\" alt=\"" + self.images[0].caption + "\" />\n<p>" + self.images[0].caption + "</p>\n</div>\n"

        elif len(self.images)>1:
            HTML="<div class=\"img_box\" style=width:" + str(self.imgWidth)+"""px;">
<div style="float: left; width: 100px;">
<a href="javascript:;" onclick="previousImage("""+ counter + ');" id="previousBut' + counter + """" style="display:none;">previous</a>
</div>
<div style="float: right; width: 100px; text-align:right;">
<a href="javascript:;" onclick="nextImage(""" + counter + ');" id="nextBut' + counter + """" style="display:inline;">next</a>
</div>
<br style="clear:both;" />
<br />
<img src=""" + "\"" + self.images[0].url + "\"  id=\"mainPic" + counter + "\" alt=\"" + self.images[0].caption + "\" />\n<p id=\"imgAttr" + counter + "\">" + self.images[0].caption + "</p>\n</div>\n"

        else:
            HTML = ""

        return HTML



class slide(object):
    "gibbon is a goat"
    def __init__(self, dirName, uplist, text):
        self.dirName= dirName
        self.uplist=uplist[:-1]
        self.up = uplist[-2]
        self.title=uplist[-1]

        if text[0]=="#":
            self.name = text[:text.find("\n")].replace("#","").strip()
            self.text = text[text.find("\n"):]
        else:
            self.name="none"
            self.text = text


        self.text = processThumbnails(self.text)


        self.imageSets=[]
        while(self.text.find("img:") > 0):
            imgStart = self.text.find("img:")
            imgStop = self.text[imgStart:].find("\n\n")+imgStart
            self.imageSets.append(imageSet(self.text[imgStart:imgStop],self.dirName))
            self.text = self.text[:imgStart]+"\n\nimageSet" + str(len(self.imageSets)-1) + "\n\n" + self.text[imgStop:]




        dummy = self.text.find("##Copyright")
        if dummy > 0:
            self.copyright=self.text[dummy+11:]
            self.startCopyright = dummy
        else:
            self.copyright=""
            self.startCopyright = len(self.text)


        dummy = self.text.find("##Further Details")
        if dummy > 0:
            self.details = self.text[dummy+17:self.startCopyright]
            self.text=self.text[:dummy]
        else:
            self.details = ""
            self.text=self.text[:self.startCopyright]

        self.previousList=[]

        self.footnotes = []

        counter = 1

        while(re.search("\^\[[A-Za-z0-9]*\]",self.text)):
            footSearch = re.search("\^\[[A-Za-z0-9]*\]",self.text)
            self.footnotes.append(footSearch.group()[2:-1])
            num = str(counter)
            self.text = self.text[:footSearch.start()] + "<sup class=\"details\"><a href=\"#cite_note_" + num + "\" id =\"cite_"+ num + "\">[" + num + "]</a></sup>" + self.text[footSearch.end():]
            counter+=1



        self.references = []

        while(re.search("\[[A-Za-z0-9]*\]:",self.details)):
            refSearch = re.search("\[[A-Za-z0-9]*\]:",self.details)
            reference = refSearch.group()[1:-2]
            self.references.append(reference)
            if reference in self.footnotes:
                num = str(self.footnotes.index(reference)+1)
                # For some reason if we don't start with a space Markdown makes a new line here
                self.details = self.details[:refSearch.start()] +" <a href=\"#cite_" + num + "\" id=\"cite_note_" + num + "\">[" + num + "]</a>" +  self.details[refSearch.end():]
            else:
                self.details = self.details[:refSearch.start()] +"[" + reference + "]" +  self.details[refSearch.end():]



        if set(self.references)!=set(self.footnotes):
            print "in slide " + self.name + " the footnotes don't match the references"
            print "references:"
            print self.references
            print "footnotes:"
            print self.footnotes




    def toHTML(self,fileName):
        javascript = """
<script type="text/javascript" src="../jquery-1.5.2.js"></script>




<script type="text/javascript">
	currentImage=["""
        for item in self.imageSets:
            javascript+="0,"

        if javascript[-1:]==",":
            javascript=javascript[:-1]

        javascript+="];\n        imageSrcs=["""
        for item in self.imageSets:
            javascript+=item.JSlistURLs() + ","

        if javascript[-1:]==",":
            javascript=javascript[:-1]

        javascript+= "];\n        captions=["

        for item in self.imageSets:
            javascript+=item.JSlistCaptions() + ","

        if javascript[-1:]==",":
            javascript=javascript[:-1]

        javascript+= """];

	function nextImage(imgGroup){
	document.getElementById("mainPic"+imgGroup).src = imageSrcs[imgGroup][currentImage[imgGroup]+1];
        document.getElementById('imgAttr'+imgGroup).innerHTML = captions[imgGroup][currentImage[imgGroup]+1];

	currentImage[imgGroup]+=1;

	nextBut = document.getElementById("nextBut"+imgGroup);
	previousBut = document.getElementById("previousBut"+imgGroup);
	if(currentImage[imgGroup]+1==imageSrcs[imgGroup].length){
	nextBut.style.display="none";
	}
	if(currentImage[imgGroup]>0){
	previousBut.style.display="inline";
}

}
	function previousImage(imgGroup){
	document.getElementById("mainPic"+imgGroup).src = imageSrcs[imgGroup][currentImage[imgGroup]-1];
        document.getElementById('imgAttr'+imgGroup).innerHTML = captions[imgGroup][currentImage[imgGroup]-1];
	currentImage[imgGroup]-=1;
	nextBut = document.getElementById("nextBut"+imgGroup);
	previousBut = document.getElementById("previousBut"+imgGroup);
	if(currentImage[imgGroup]==0){
	previousBut.style.display="none";
	}
	if(currentImage[imgGroup]< imageSrcs[imgGroup].length-1){
        nextBut.style.display="inline";
}

}
</script>


<script type="text/javascript">
$(document).ready(function() {
   $("a#lnkDetails").click(function(event) {
   event.preventDefault();
    $(".details").toggle("slow");
   });
   $("a#lnkCopyright").click(function(event) {
   event.preventDefault();
    $("#copyright").toggle();
   });
 });

</script>"""

        upListHTML = "> "
        for item in self.uplist:
            upListHTML+= "<a href=\"../" + item.replace(" ","-").lower() + ".html\">" + item + "</a>"


        upButton="<div id=\"up\">\n<a href=\"../" + self.up.replace(" ","-").lower() + ".html\"><img src=\"../arrow-up-50.jpg\" /></a>\n</div>"


        position=self.fullList.index(self.name)

        nextList=[]
        previousList=[]

        for counter in range(0,position):
            previousList.append(self.fullList[counter])

        for counter in range(position+1,len(self.fullList)):
            nextList.append(self.fullList[counter])


        buttons = """
<div id="previousContainer">
<div id="innerPreviousContainer">
"""

        if previousList!=[]:
            buttons+="<ul id=\"prevList\">\n"
            for i in range(0,len(previousList)):
                buttons+="<li><a href=\"" + previousList[i].replace(" ","-").lower() + ".html\">" + str(i+1) + ". " + previousList[i] + "</a></li>\n"

            buttons+="</ul>\n<a href=\"" + previousList[-1].replace(" ","-").lower() + ".html\"><img src=\"../arrow-left-80.jpg\" /></a>"

        buttons+="\n</div>\n</div>\n\n<div id=\"nextContainer\">\n\n"

        if nextList!=[]:
            buttons+="<a href=\"" + nextList[0].replace(" ","-").lower() + ".html\"><img src=\"../arrow-right-80.jpg\" /></a>\n"
            buttons+="<ul id=\"nextList\">\n"
            for i in range(0,len(nextList)):
                buttons+="<li><a href=\"" + nextList[i].replace(" ","-").lower() + ".html\">" + str(position+i+2) + ". " + nextList[i] + "</a></li>\n"

            buttons+="</ul>\n"
        else:
            #add an up button
            buttons+="<a href=\"../" + self.up.replace(" ","-").lower() + ".html\"><img src=\"../arrow-up-80.jpg\" /></a>\n"
            buttons+="<ul id=\"nextList\">\n"
            buttons+="<li><a href=\"../" + self.up.replace(" ","-").lower() + ".html\">up to " +  self.up + "</a></li>\n"
            buttons+="</ul>\n"




        buttons+="</div>"

        bulk="""
<div id="box">

<h1>"""
        bulk+= self.title + ": "

        #no number if only one slideshow
        if len(self.fullList) > 1:
            bulk +=  str(position+1) + ". "

        bulk+= self.name

        bulk+="""</h1>
"""

        bulk+= markdown.markdown(self.text)

        bulk+="""

<br style="clear:both;" />
"""
        if self.details!="":
            bulk+="<div class=\"details\">\n<h2>Further Details</h2>\n"
            bulk+=markdown.markdown(unicode("\n" + self.details + "\n",errors="ignore"))

            bulk+="""
<br style="clear:both;" />
</div>
"""

        if self.copyright!="" or self.globalCopyright!="":
            bulk+="<div id=\"copyright\">\n<h2>Copyright</h2>\n"
            bulk+=markdown.markdown("\n" + self.globalCopyright + "\n")
            bulk+=markdown.markdown("\n" + self.copyright + "\n")
            bulk+="""
<br style="clear:both;" />
</div>"""

        bulk+="""
</div>
<div id="bottom">
"""
        if self.details!="":
            bulk+="""
<div id="detailsContainer">
<a href="#" id="lnkDetails">Further details</a>
</div>
"""
        bulk+="\n<div id=\"copyrightContainer\">\n<a href=\"../comment.html\">Improve</a>"

        if self.copyright!="" or self.globalCopyright!="":
            bulk+=" | <a href=\"#\" id=\"lnkCopyright\">Copyright</a>"
        bulk+="""\n</div>

<br style="clear:both;" />

</div>
"""


        for counter in range(0,len(self.imageSets)):
            bulk = bulk.replace("<p>imageSet"+str(counter)+"</p>",self.imageSets[counter].toHTML(str(counter)))




        temp = Template(open("temp.html").read())

        values = {
            "topTitle" : self.title + " - " + self.name,
            "javascript" : javascript,
            "uplist" : upListHTML,
            "up" : upButton,
            "relativeURL" : "../",
            "buttons" : buttons,
            "bulk" : bulk,
            "statcounterText" : statcounterText
            }

        outputFile = open(fileName, 'w')
        outputFile.write(temp.substitute(values))
        outputFile.close()

        return 0



class slideShow(object):
    "slideshow object"
    def __init__(self,filePath):

        inputFile = open(filePath, 'r')

        upLine = inputFile.readline()

        if upLine[:1]!=">":
            print "need up line"
            sys.exit(0)

        self.upList = []

        for item in upLine[1:].split(">"):
            self.upList.append(item.strip())

        print self.upList
        self.title = self.upList[-1]
        print self.title


        self.dirPath = filePath[:-4]+"/"

        if not os.path.isdir(self.dirPath):
            print "need to make dir "+ self.dirPath
            os.mkdir(self.dirPath)


        text = ""
        self.slides=[]
        ignore = 0
        for line in inputFile:
            if line[0]=="%":
                ignore = 1-ignore
                continue
            if not ignore:
                if line[0]=="#" and line[1]!="#":
                    if text.strip()!="":
                        self.slides.append(slide(self.dirPath,self.upList,text.strip()+"\n\n"))
                        text = ""
                text+=line



        self.globalCopyright=""

        if text[:18]=="#Global Copyright\n":
            self.globalCopyright = text[17:]
        else:
            self.slides.append(slide(self.dirPath,self.upList,text.strip()+"\n\n"))


        inputFile.close()

        dummyList = []

        for counter in range(0,len(self.slides)):
            dummyList.append(self.slides[counter].name)

        for counter in range(0,len(self.slides)):
            self.slides[counter].fullList = dummyList[:]

            self.slides[counter].globalCopyright=self.globalCopyright


    def toHTML(self):

        for item in self.slides:
            item.toHTML(item.dirName + item.name.replace(" ","-").lower() + '.html')

        self.slides[0].toHTML(self.slides[0].dirName+ "index.html")





def processSlideShow(filePath):


    aSlideShow = slideShow(filePath)

    aSlideShow.toHTML()




def findNewFiles():
    #returns list of files in directory
    dirList = os.listdir("./")
    result=[]
    for name in dirList:
        if name[-4:]==".mar":
            # returns number of seconds since epoch
            marTime = os.path.getmtime(name)
            if os.path.isfile(name[:-4]+"/index.html"):
                if os.path.getmtime(name[:-4]+"/index.html") < marTime:
                    print name + " needs updating"
                    result.append(name)
            elif os.path.isfile(name[:-4]+".html"):
                if os.path.getmtime(name[:-4]+".html") < marTime:
                    print name + " needs updating"
                    result.append(name)
            else:
                print name + " not processed yet"
                result.append(name)
    return result


def findAllFiles():
    #returns list of files in directory
    dirList = os.listdir("./")
    result=[]
    for name in dirList:
        if name[-4:]==".mar":
            result.append(name)
    return result


def processList(listOfFiles):
    for fileName in listOfFiles:
        if fileName=="index.mar":
            print fileName + " is the index file"
            processIndex()
        else:
            inputFile = open(fileName, 'r')
            title = inputFile.readline()
            inputFile.close()
            if title.count(">")<1:
                print fileName + " doesn't have a > in the first line"
            elif title.count(">")==1:
                print fileName + " is top-level"
                processTopLevel(fileName)
            else:
                print fileName + " is a slideshow"
                processSlideShow(fileName)


if __name__ == "__main__":


    allOpt = False
    statcounter = False


    try:
        options, arguments = getopt.gnu_getopt(sys.argv[1:],
        'has', ['help'])
    except getopt.error:
        print 'error: you tried to use an unknown option'
        sys.exit(0)

    for o,a in options:
        if o in  ('-h','--help'):
            print __doc__
            sys.exit(0)
        elif o=='-a':
            print "processing all"
            allOpt = True
            filesToProcess = findAllFiles()
        elif o=='-s':
            print "using statcounter"
            statcounter = True

    if not statcounter:
        statcounterText = ""

    if not allOpt:
        if len(arguments) == 0:
            print 'processing all new .mar files'
            filesToProcess = findNewFiles()
        else:
            for argument in arguments:
                if argument[-4:]!=".mar":
                    print argument + ' is not a .mar file'
                    sys.exit(0)
                if not os.path.isfile(argument):
                    print argument + ' does not exist'
                    sys.exit(0)
            print 'processing specified files'
            filesToProcess = arguments

    processList(filesToProcess)
